;************************************************
;*****                                      *****
;*****   HYPERTAPE DUMP AND LOAD ROUTINES   *****
;*****   ADAPTED FROM JUNIOR'S MONITOR      *****
;*****    WRITTEN BY A.NACHTMANN  1980      *****
;*****                                      *****
;************************************************
;
 .CE
 .BA $9DF
 .MC $A9DF
 .OS
;
;
;
;ZERO PAGE
;
POINTL .DE $FA ;ADDRESS POINTER
POINTH .DE POINTL+1
;
;DATA BUFFERS
;
SY .DE $1A69 ;SYN COUNTER
BYTE .DE SY+1 ;BYTE FROM TAPE
CHAR .DE BYTE+1 ;CHARACTER FROM TAPE
HIGHER .DE CHAR+1 ;3600 Hz HALF PERIODE DELAY
LOWER .DE HIGHER+1 2400 Hz HALF PERIODE DELAY
CHKL .DE LOWER+1 ;CHECKSUM
CHKH .DE CHKL+1
SAL .DE CHKH+1 ;START ADDRESS
SAH .DE SAL+1
EAL .DE SAH+1 ;END ADDRESS
EAH .DE EAL+1
SYNCNT .DE EAH+1 ;SYN COUNTER
BITS .DE SYNCNT+1 ;# OF BITS
FIRST .DE BITS+1 ;HALF PERIODE 3600 Hz
SECOND .DE FIRST+1 ;HALF PERIODE 2400 Hz
GANG .DE SECOND+1 ;TEMP FOR PBD-BITS
ID .DE GANG+1 ;FILE ID
;
;PIA ADDRESSES
;
PAD .DE $1A80 ;PORT A DATA
PADD .DE PAD+1 ;PORT A DIRECTION
PBD .DE $1A82 ;PORT B DATA
PBDD .DE PBD+1 ;PORT B DIRECTION
;
;TIMER
;
CNTA .DE $1AF4 ;CLK1T IRQ DISABLED
CNTC .DE $1AF6 ;CLK64T IRQ DISABLED
RDFLAG .DE $1AD5 ;TIMER FLAG = BIT 7
RDTDIS .DE $1AD4 ;READ TIMER IRQ DISABLED
;
;
;
;********* DUMP/DUMPT *********
;
DUMP LDA #125 ;3600 Hz HALF PERIODE DELAY
 STA HIGHER
 LDA #195 ;2400 Hz HALF PERIODE DELAY
 STA LOWER
 LDA #3 ;TOGGLE 3 TIMES AT 3600 Hz
 STA FIRST
 LDA #2 ;TOGGLE 2 TIMES AT 2400 Hz
 STA SECOND
;
DUMPT LDA #%01000111 PBD PATTERN
 LDX #%11111111 PBDD PATTERN
 STA PBD ;INPUT OFF, OUTPUT ON,
;                       STROBE DISABLED
 STA GANG ;SAVE BITS OF PBD
 STX PBDD ;PB0...PB7 IS OUTPUT
 LDA #%00000000 PAD PATTERN
 LDX #%01111111 PADD PATTERN
 STA PAD ;SEGMENTS ON BUT DISABLED
 STX PADD ;ALL OUTPUT EXCEPT PA7
 STA CHKL ;RESET CHECKSUM
 STA CHKH
 LDA SAL ;INIT DUMPT POINTER
 STA *POINTL
 LDA SAH
 STA *POINTH
 LDX #255 ;SET SYNC COUNTER
 STX SYNCNT
;
SYNCS LDA #$16 ;ASCII SYN CHAR
 JSR OUTCH ;OUTPUT 255 SYNC'S
 DEC SYNCNT
 BNE SYNCS
;
 LDA #'* ;OUTPUT START CHARACTER
 JSR OUTCH
 LDA ID ;DUMP ID
 JSR OUTBT ;NO CHECKSUM COMPUTATION
 LDA SAL ;DUMP SA
 JSR OUTBTC ;START CHECKSUM COMPUTATION
 LDA SAH
 JSR OUTBTC
;
DATATR LDA *POINTH
 CMP EAH ;ENTIRE FILE DUMPED?
 BNE HEXDAT
 LDA *POINTL
 CMP EAL
 BNE HEXDAT
;
 LDA #'/ ;END OF DATA CHAR
 JSR OUTCH ;STOP CHECKSUM COMPUTATION
 LDA CHKL ;DUMP CHECKSUM
 JSR OUTBT
 LDA CHKH
 JSR OUTBT
 LDA #$04 ;ASCII EOT CHAR
 JSR OUTCH ;MARKS END OF FILE
 LDA #$04
 JSR OUTCH
 RTS
;
HEXDAT LDY #0
 LDA (POINTL),Y ;FETCH DATA BYTE
 JSR OUTBTC ;DUMP IT
 INC *POINTL ;SET POINTER ON
 BNE DATATR ;NEXT DATA BYTE
 INC *POINTH
 JMP DATATR
;
;
;OUTBTC DUMPS A BYTE AS 2 ASCII'S AND
;COMPUTES CHECKSUM
;
OUTBTC TAY ;SAVE DATA BYTE
 CLC
 ADC CHKL ;COMPUTE CHECKSUM
 STA CHKL
 LDA CHKH
 ADC #0
 STA CHKH
 TYA ;GET DATA BYTE
;
;OUTBT IS THE ENTRY FOR DUMP WITHOUT
;CHECKSUM COMPUTATION
;
OUTBT TAY ;SAVE DATA BYTE
 LSR A ;GET UPPER NIBBLE
 LSR A
 LSR A
 LSR A
 JSR NIBOUT ;DUMP IT AS ASCII
 TYA ;GET DATA BYTE
 AND #%00001111 GET LOWER NIBBLE
 JSR NIBOUT
 RTS
;
;NIBOUT DUMPS A NIBBLE AS AN ASCII
;
NIBOUT CMP #$A ;CONVERT TO ASCII
 CLC
 BMI NIB
 ADC #7
NIB ADC #'0
;
;OUTCH IS THE ENTRY FOR DUMPING ASCII'S
;
OUTCH LDX #8 ;SET UP FOR 8 BITS
 STX BITS
ONE LSR A ;SHIFT BITS OUT
 PHA ;SAVE CHARACTER
 BCC ZERO
 JSR HIGH ;1=HIGH-LOW-LOW
 JSR LOW
 JSR LOW
 JMP ZRO
;
ZERO JSR HIGH ;0=HIGH-HIGH-LOW
 JSR HIGH
 JSR LOW
;
ZRO PLA ;GET CHAR AGAIN
 DEC BITS ;ALL BITS DUMPED?
 BNE ONE
 RTS
;
;HIGH :3 PERIODS OF 3600 Hz
;
HIGH LDX FIRST ;AMOUNT OF HALF PERIODS
HIG BIT RDFLAG ;TIME OUT?
 BPL HIG
 LDA HIGHER ;SET UP TIMER FOR SHORT PERIOD
 STA CNTA ;IRQ DISABLED, CLK1T
 LDA GANG
 EOR #%10000000 TOGGLE OUTPUT
 STA PBD
 STA GANG ;SAVE STATUS QUO
 DEX
 BNE HIG
 RTS
;
;LOW: 2 PERIODS OF 2400 Hz
;
LOW LDX SECOND ;AMOUNT OF HALF PERIODS
LO BIT RDFLAG ;TIME OUT
 BPL LO
 LDA LOWER ;SET UP TIME FOR LONG PERIODE
 STA CNTA
 LDA GANG
 EOR #%10000000 TOGGLE
 STA PBD
 STA GANG
 DEX
 BNE LO
 RTS
;
;********* RDTAPE *********
;
; ID=0: IGNORE ID ON TAPE; DISCARD DATA BLOCK
; AT SA COMING FROM TAPE.
;
; ID=FF: IGNORE ID ON TAPE; DISCARD DATA BLOCK
; AT SA STORED IN MEMORY, IGNORE SA ON TAPE.
;
RDTAPE LDA #%00110010 INPUT RECORDER ON
 STA PBD ;OUTPUT RECORDER OFF, PB7=IN
 STA GANG ;STROBE '9' IS ENABLED
 LDA #%01111110 PB0 IS INPUT
 STA PBDD ;PB7 IS INPUT
 LDA #%01111111 PA0...PA6 IS OUTPUT
 STA PADD ;SEGMENTS ON
 LDA #0
 STA CHKL ;RESET CHECKSUM
 STA CHKH
;
SYNC LDA #$FF ;RESET FOR SYNC
 STA CHAR
;
SYNCA JSR RDBIT ;READ A BIT FROM TAPE
 ROR CHAR ;RIGHT SHIFT
 LDA CHAR ;GET CURRENT CHARACTER
 JSR BETWEEN ;DISPLAY NOT SYN CHAR
 CMP #$16 ;ASCII SYN?
 BNE SYNCA ;IF NOT RESYNC
 LDY #10 ;TRY IT FOR 10 SYNS AT LEAST
 STY SY ;SYN COUNTER
;
TENSYN JSR RDCH
 JSR CHARVU ;DISPLAY SYN CHAR
 CMP #$16 ;STILL SYN?
 BNE SYNC ;IF NOT RETURN
 DEC SY ;10 SYNS RECEIVED?
 BNE TENSYN ;RETURN WHILE NOT TEN SYNS
;
STAR JSR RDCH ;WAIT FOR '*' CHAR
 JSR CHARVU
 CMP #'* ;START OF DATA?
 BEQ STARA
 CMP #$16 ;STILL SYN?
 BEQ STAR ;THEN WAIT FOR *
 BNE RDTAPE ;ELSE START OVER
;
STARA JSR CHARVU ;DISPLAY *
 JSR RDBYT ;READ ID FROM TAPE
 CMP ID ;REQUESTED ID?
 BNE CHKID ;SPECIAL ID REQUESTED?
;
RDSA JSR RDBYT ;READ SAL FROM TAPE
 JSR CHKSUM ;CHECKSUM COMPUTATION
 STA *POINTL ;SET UP STORE POINTER
 JSR RDBYT ;READ SAH
 JSR CHKSUM
 STA *POINTH
;
FILMEM JSR RDBYT ;READ DATA BYTE
 BMI RDTAPE ;NOT HEX? THEN RESTART
 BEQ CHECK ;END OF DATA CHAR?
 JSR CHKSUM
 LDY #0
 STA (POINTL),Y STORE BYTE IN MEMORY
 INC *POINTL ;SET POINTER ON
 BNE FMA ;NEXT DATA BYTE
 INC *POINTH
;
FMA JSR VU ;DISPLAY *
 JMP FILMEM ;READ NEXT DATA BYTE
;
CHECK JSR RDBYT ;READ CHECKSUM FROM TAPE
 CMP CHKL ;AND COMPARE IT
 BNE SYNVEC
 JSR RDBYT
 CMP CHKH
 BNE SYNVEC
 RTS ;RETURN
;
SYNVEC JMP RDTAPE
;
CHKID LDA ID
 CMP #$00 ;ID = 00?
 BEQ RDSA ;IGNORE ID
 CMP #$FF ;ID = FF?
 BNE SYNVEC ;THEN RESTART
 JSR RDBYT ;READ SA, BUT IGNORE IT
 JSR CHKSUM
 JSR RDBYT
 JSR CHKSUM
 LDA SAL ;USE STORED SA
 STA *POINTL
 LDA SAH
 STA *POINTH
 JMP FILMEM
;
;RDBIT READS 1 BIT FROM TAPE
;CARRY = BIT FOUND
;
RDBIT BIT PBD ;3600 Hz? (=1 FROM PLL)
 BPL RDBIT
 LDA RDTDIS ;GET COUNT DOWN
 LDY #255
 STY CNTC ;2400 Hz COUNT DOWN
 LDY #20
;
RDBA DEY ;DELAY JITTER TIME
 BNE RDBA
;
RDBB BIT PBD ;2400 Hz?
 BMI RDBB
 SEC
 SBC RDTDIS ;SET OR RESET C-FLAG
 LDY #255
 STY CNTC ;3600 Hz COUNT DOWN
 LDY #7 ;DELAY FOR JITTER
;
RDBC DEY
 BNE RDBC
 RTS
;
;BETWEEN DIPLAYS A PATTERN ON THE 7-SEGMENTS
;DISPLAY
;
BETWEEN PHA ;SAVE CHAR
 LDA #%00110110 OUTPUT BETWEEN CHAR
 STA PAD
 PLA
 JSR VU ;DON'T CHANGE BETWEEN CHAR
 RTS
;
;RDBYT READS 1 BYTE=TWO HEX CHARS FROM TAPE
;AND RETURNS IT IN ACCU
;N=1: NOT VALID HEX CHARACTER
;Z=1: END OF DATA TRANSMISSION
;Z=N=0: VALID BYTE IN ACCU
;
RDBYT JSR RDCH ;READ ASCII CHARACTER FROM TAPE
 CMP #'/ ;END OF DATA CHAR?
 BNE RBB
;
RBA RTS ;ERROR EXIT
;
RBB JSR ASCHEX ;ASCII TO HEX CONVERSION
 BMI RBA ;NOT VALID CHARACTER
 ASL A ;SHIFT NIBBLE TO LEFT
 ASL A
 ASL A
 ASL A
 STA BYTE ;SAVE HIGH ORDER NIBBLE
 JSR RDCH ;READ NEXT CHARACTER
 CMP #'/ ;END OF DATA CHARACTER
 BEQ RBA
 JSR ASCHEX ;ASCII TO HEX CONVERSION
 BMI RBA
 ORA BYTE ;ASSEMBLE BYTE
 LDY #1 ;Z=0
 RTS ;NORMAL EXIT
;
;ASCHEX CONVERTS AN ASCII CHAR IN A HEX NIBBLE
;AND RETURNS IT IN ACCU.
;N=1: IF NOT VALID HEX CHARACTER
;
ASCHEX CMP #'0
 BMI NOTVAL
 CMP #': ;'9'+1
 BMI VALID
 CMP #'A
 BMI NOTVAL
 CMP #'G ;'F'+1
 BMI VALID
;
NOTVAL LDY #255 ;SET N-FLAG
 RTS
;
VALID CMP #'@
 BMI VAL
 CLC
 ADC #9
;
VAL AND #%00001111 HEX IS LOWER NIBBLE
 RTS
;
;RDCH READS AN ASCII FROM TAPE AND RETURNS IT
;IN ACCU
;
RDCH LDX #8 ;SET UP FOR 8 BITS
;
READ JSR RDBIT ;READ A BIT FROM TAPE
 ROR CHAR ;SHIFT BIT INTO CHAR
 DEX ;ALL BITS RECEIVED?
 BNE READ
 ROL CHAR ;B7 MUST BE ZERO
 LSR CHAR
 LDA CHAR ;STORE IT IN ACCU
 RTS
;
CHKSUM PHA ;SAVE CHAR
 CLC
 ADC CHKL ;CHK +:= BYTE
 STA CHKL
 LDA CHKH
 ADC #0
 STA CHKH
 PLA ;RESTORE CHAR
 RTS
;
;
;CHARVU PUTS A PATTERN ON 7-SEGMENTS DISPLAY
;CORRESPONDING TO THE RECEIVED CHARACTER
;VU IS THE ENTRY TO FLIP-FLOP THE DISPLAYS
;WITHOUT CHANGING THE PATTERN.
;
CHARVU PHA ;SAVE CHAR
 EOR #$7F ;INVERTED TO SEGMENTS
 STA PAD ;
 PLA ;RESTORE CHAR
;
VU PHA
 LDA GANG
 EOR #2 ;EXCHANGE DISPLAYS
 STA PBD
 STA GANG ;SAVE STATUS QUO
 PLA
 RTS
;
 .EN
